% ------------------------------------------------------------------------
% This file is part of the EHCM Toolbox v1
%
% The EHCM Toolbox - Energy Hub Component Modeling for Model Predictive Control.
% Copyright (C) 2015  Automatic Control Laboratory, ETH Zurich.
%
% The EHCM Toolbox is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% The EHCM Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the EHCM Toolbox.  If not, see <http://www.gnu.org/licenses/>.
%
% ------------------------------------------------------------------------

function con = EHConvConstraints(discSS,x,u,Np)
% con = buildingConstraints(bdmodel,x0,u,v,Np)
% constraints depend on the current state (linearize around the initial)
% and disturbance vector.

%% Initial time constraints on states do not apply, only constraints on inputs!
Fx = discSS.constraints.Fx;
Fu = discSS.constraints.Fu;
g = discSS.constraints.g;

%% regular constraints for the next steps
for ii = 2:Np
    % get the matrices
    Fx = blkdiag(Fx,discSS.constraints.Fx);
    Fu = blkdiag(Fu,discSS.constraints.Fu);
    g = [g;discSS.constraints.g];
end

xf = [];
uf = [];
for ii = 1:Np
    xf = [xf;x(:,ii)];
    uf = [uf;u(:,ii)];
end



% get the constraints set over the prediction horizon
con = Fx*xf +Fu*uf <= g;

end