% ------------------------------------------------------------------------
% This file is part of the EHCM Toolbox v1
%
% The EHCM Toolbox - Energy Hub Component Modeling for Model Predictive Control.
% Copyright (C) 2015  Automatic Control Laboratory, ETH Zurich.
%
% The EHCM Toolbox is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% The EHCM Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the EHCM Toolbox.  If not, see <http://www.gnu.org/licenses/>.
%
% ------------------------------------------------------------------------

function distVecBd = getDistVec(EHComp,disturb,disturbVar,sim,Np)
%% function verified that works properly
if Np>60
    error('Too large prediction horizon (conflict with the available disturbance predictions');
end

%%
bdvIdent = EHComp.discreteTimeModel.identifiers.v;   % disturbance identifiers of the building
vIdent = disturb.identifiers;   % disturbance identifiers of the disturbance file

nbdDist = size(bdvIdent,1); % number of building disturbances
nDist = size(vIdent,1); % number of different disturbances

% create the desired output disturbance vectors
vreal = zeros(sim.horizon,nbdDist); % realizations of the buildings disturbances over the horizon
vpred = zeros(sim.horizon,Np,nbdDist);  % prediction vector for every distrubance of the bulding

% same variables for the disturbances, since they are the same disturbances
% for the different buildings and devices!
vvar = cell(sim.horizon,1);
for ii = 1:sim.horizon
    vvar{ii} = sdpvar(Np,nbdDist);
end
% assign the correct vectors to the buildings and components devices
for ii = 1:nDist % iterate over all possible disturbances
    if iscell(vIdent{ii}) % check if the disturbance is unique (Tgnd , AHU_Twb) are the same
        
        for jj = 1:length(vIdent{ii})
            tv = vIdent{ii}{jj};
            tVec = cellfun(@(x) strncmpi(x,tv,length(tv)),bdvIdent);
            idx = find(tVec==1);
            for kk = 1:length(idx)
                vreal(:,idx(kk)) = disturb.vreal(sim.init:sim.init+sim.horizon-1,ii);
                
                for ll = 1:sim.horizon % we have not prediction for the first step.
                    kt = sim.init + (ll-1);
                    rs = floor((kt-2)/12)+1;
                    cs = mod((kt-2),12)+1;
                    vpred(ll,:,idx(kk)) = disturb.vpred(rs,cs:cs+Np-1,ii);
                    vvar{ll}(:,idx(kk)) = disturbVar(ii,cs:cs+Np-1);
                end
            end
        end
        
    else
        % get the identifier of the disturbance and chech to which
        % disturbances of the building is referred to
        tv = vIdent{ii};
        tVec = cellfun(@(x) strncmpi(x,tv,length(tv)),bdvIdent);
        idx = find(tVec==1);
        
        % assign the disturbance realization and prediction vector to the
        % proper position of the disturbance vector for the building.
        for kk = 1:length(idx)
            % realizations for the simulation interval that we need
            vreal(:,idx(kk)) = disturb.vreal(sim.init:sim.init+sim.horizon-1,ii);
            
            % predictions for the simulation interval that we need
            for ll = 1:sim.horizon % we have not prediction for the first step.
                kt = sim.init + (ll-1);
                rs = floor((kt-2)/12)+1;
                cs = mod((kt-2),12)+1;
                vpred(ll,:,idx(kk)) = disturb.vpred(rs,cs:cs+Np-1,ii);
                vvar{ll}(:,idx(kk)) = disturbVar(ii,cs:cs+Np-1);
            end
        end
    end
end

distVecBd.vreal = vreal;
distVecBd.vpred = vpred;
distVecBd.vvar = vvar;
end