% ------------------------------------------------------------------------
% This file is part of the EHCM Toolbox v1
%
% The EHCM Toolbox - Energy Hub Component Modeling for Model Predictive Control.
% Copyright (C) 2015  Automatic Control Laboratory, ETH Zurich.
%
% The EHCM Toolbox is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% The EHCM Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the EHCM Toolbox.  If not, see <http://www.gnu.org/licenses/>.
%
% ------------------------------------------------------------------------

function details = getModelMatrices(F,h,parametricVariables)

if ~isempty(F)
    if any(is(F,'sos2'))
        error('SOS2 structures not allowed');
    end
end

[~,~,~,model] = export(F,h,sdpsettings('solver','quadprog','relax',2));
if isempty(model)
    error('System can not be derived: You may have nonlinear constraints');
end


if nargin < 3
    x = recover(model.used_variables);
    parameters = [];
else
    % Make sure they are sorted
    parameters = getvariables(parametricVariables);
    x = recover(setdiff(model.used_variables,parameters));
    notparameters = find(ismember(model.used_variables,getvariables(x)));%parameters);
    parameters = setdiff(1:length(model.used_variables),notparameters);
    if ~isempty(parameters)
        y = recover(model.used_variables(parameters));
    end
end

% Ex==f
E = model.F_struc(1:model.K.f,2:end);
f = -model.F_struc(1:model.K.f,1);
% Ax <= b
A = -model.F_struc(model.K.f+1:model.K.f+model.K.l,2:end);
b = model.F_struc(model.K.f+1:model.K.f+model.K.l,1);
c = model.c;
h = model.f;
Q = model.Q;

if ~isempty(parameters)
    b = b-A(:,parameters)*y;
    f = f-E(:,parameters)*y;
    h = h + c(parameters)'*y;
    A(:,parameters) = [];
    E(:,parameters) = [];
    c(parameters) = [];
    Q2 = model.Q(notparameters,parameters);
    c = c + 2*Q2*y;
    Q = Q(notparameters,notparameters);
end


if nnz(Q)>0
    details.info = 'min x''Qx+c''x+h';
else
    details.info = 'min c''x+h';
end
if isempty(E)
    details.info = [details.info ' s.t. Ax<=b'];
else
    details.info = [details.info ' s.t. Ax<b, Ex=f'];
end

nx = length(x);

if isempty(c)
    c = zeros(nx,1);
end
details.c = c;

if isempty(h)
    h = 0;
end
details.h = h;

if isempty(Q)
    Q = zeros(nx,nx);
end
details.Q = Q;

if isempty(A)
    A = zeros(1,nx);
end
details.A = A;

if isempty(b)
    b = 0;
end
details.b = b;

if isempty(E)
    E = zeros(1,nx);
end
details.E = E;

if isempty(f)
    f = 0;
end
details.f = f;
details.primal = x;
end