% ------------------------------------------------------------------------
% This file is part of the EHCM Toolbox v1
%
% The EHCM Toolbox - Energy Hub Component Modeling for Model Predictive Control.
% Copyright (C) 2015  Automatic Control Laboratory, ETH Zurich.
%
% The EHCM Toolbox is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% The EHCM Toolbox is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with the EHCM Toolbox.  If not, see <http://www.gnu.org/licenses/>.
%
% ------------------------------------------------------------------------


function con = buildingConstraints(discSS,x,u,v,Np)
% con = buildingConstraints(bdmodel,x0,u,v,Np)
% constraints depend on the current state (linearize around the initial)
% and disturbance vector.

%% equality constraints
x0 = x(:,1);
%  Define the extended matrices
nu = size(u,1);
Ae = discSS.discreteTimeModel.ss.A(:,:);
Bxue = [];
for ii = 1:nu
    % linearize around the current point... (residualization reduced order)
    Bxue = [Bxue,discSS.discreteTimeModel.ss.Bxu(:,:,ii)*x0];
end

Bvue = cell(Np,1);
for jj = 1:Np
    Bvue{jj} = [];
    for ii = 1:nu
        Bvue{jj} = [Bvue{jj},discSS.discreteTimeModel.ss.Bvu(:,:,ii)*v.vpred(jj,:)'];
    end
end

Bue = cell(Np,1);
for jj = 1:Np
    Bue{jj} = discSS.discreteTimeModel.ss.Bu(:,:) + Bxue + Bvue{jj};
end

Bve = discSS.discreteTimeModel.ss.Bv(:,:);

conE = [];
for ii = 1:Np
    conE = conE + [x(:,ii+1) == Ae*x(:,ii) + Bue{ii}*u(:,ii) + Bve*v.vvar(ii,:)'];
end





%% inequality constraints


% Initial time constraints on states do not apply, only constraints on inputs!
Fx = discSS.discreteTimeModel.constraints.Fx;
Fu = discSS.discreteTimeModel.constraints.Fu;
Fv = discSS.discreteTimeModel.constraints.Fv;
g = discSS.discreteTimeModel.constraints.g;

% regular inequality constraints for the next steps

for ii = 2:Np
    % get the matrices
    Fx = blkdiag(Fx,discSS.discreteTimeModel.constraints.Fx); % get the unbalanced real variables for the constraints
    Fu = blkdiag(Fu,discSS.discreteTimeModel.constraints.Fu);
    Fv = blkdiag(Fv,discSS.discreteTimeModel.constraints.Fv);
    g = [g;discSS.discreteTimeModel.constraints.g];
end

% construct the constraint set
xf = [];
uf = [];
vf = [];
for ii = 1:Np
    xf = [xf;x(:,ii)];
    uf = [uf;u(:,ii)];
    vf = [vf;v.vvar(ii,:)'];
end



% get the building constraints
conI = Fx*xf +Fu*uf + Fv*vf <= g;

con = conE + conI;
end